<?php
namespace Paybylink\Payments\Helper;

use Magento\Sales\Model\Order;

final class Data extends \Magento\Framework\App\Helper\AbstractHelper
{
    public function __construct(\Magento\Framework\App\Helper\Context $context)
    {
        parent::__construct($context);
    }

    public function createPayment(Order $order)
    {
        $shopId             = intval($this->getConfigData('pbl_shopid'));
        $description        = strval('Zamówienie #' . $order->getId());
        $control            = strval('#' . $order->getId());
        $email              = strval($order->getCustomerEmail());
        $price              = floatval($order->getGrandTotal());
        $notifyURL          = strval($this->_urlBuilder->getUrl('paybylink/checkout/notify'));
        $returnUrlSuccess   = strval($this->_urlBuilder->getUrl('paybylink/checkout/success'));
        $hash               = $this->getConfigData('pbl_hash');
    
        $data   = $hash . "|" . $shopId . "|" . sprintf("%.2f", $price);
    
        if ($control != null && $control != "") {
            $data .= "|" . $control;
        }
    
        if ($description != null && $description != "") {
            $data .= "|" . $description;
        }
        
        if ($email != null && $email != "") {
            $data .= "|" . $email;
        }
    
        if ($notifyURL != null && $notifyURL != "") {
            $data .= "|" . $notifyURL;
        }
    
        if ($returnUrlSuccess != null && $returnUrlSuccess != "") {
            $data .= "|" . $returnUrlSuccess;
        }
    
        $signature = hash('sha256', $data);
    
        $paybylinkData  = [
            'shopId'            => $shopId,
            'price'             => $price,
            'control'           => $control,
            'description'       => $description,
            'email'             => $email,
            'notifyURL'         => $notifyURL,
            'returnUrlSuccess'  => $returnUrlSuccess,
            'signature'         => $signature
        ];
    
        $data_string = json_encode($paybylinkData);
    
        $ch = curl_init('https://secure.pbl.pl/api/v1/transfer/generate');                                                                      
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");                                                                    
        curl_setopt($ch, CURLOPT_POSTFIELDS, $data_string);                                                                  
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);                                                                      
        curl_setopt($ch, CURLOPT_HTTPHEADER, array(                                                                          
            'Content-Type: application/json',                                                                                
            'Content-Length: ' . strlen($data_string))                                                                      
        );    
    
        $result = curl_exec($ch);
        $result = json_decode($result);
    
        if(isset($result->transactionId) AND !empty($result->transactionId)) {
            $payURL = 'https://secure.pbl.pl/transfer/' . $result->transactionId;
        }

        return $result;
    }

    public function getConfigData($configPath)
    {
        return $this->scopeConfig->getValue(
            'payment/paybylink_payments/' . $configPath,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
    }

}
