<?php
namespace Paybylink\Payments\Helper;

use Magento\Sales\Model\Order;

final class Notify extends \Magento\Framework\App\Helper\AbstractHelper
{
    
    public function process($data)
    {
        if(!empty($data)) {
            $response = json_decode($data, true);

            $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
            $orderFactory = $objectManager->get('Magento\Sales\Model\OrderFactory');
            $order = $orderFactory->create()->loadByAttribute('entity_id', str_replace('#', '', $response['control']));

            $pbl_data = array(
                'transactionId' => $response['transactionId'],
                'control' => $response['control'],
                'email' => $response['email'],
                'amountPaid' => $response['amountPaid'],
                'notificationAttempt' => $response['notificationAttempt'],
                'paymentType' => $response['paymentType'],
                'apiVersion' => $response['apiVersion'],
                'signature' => $response['signature']
            );

            if(!$this->checkSignature($this->getConfigData('pbl_hash'), $pbl_data)) {
                exit('Wrong checksum' . json_encode($response));
            }

            $this->complete($order, $response);

            exit('OK');
        } else {
            exit('Empty data');
        }
    }

    protected function checkSignature($secret, $data){
        $sign = sprintf("%s|%s|%s|%s|%.2f|%d|%s|%d", $secret, $data['transactionId'], $data['control'], $data['email'], $data['amountPaid'], $data['notificationAttempt'], $data['paymentType'], $data['apiVersion']);
        $hash = hash('sha256', $sign);
        return $hash == $data['signature'];
    }

    private function complete($order, $paymentData)
    {
        $payment = $order->getPayment();
        $payment->setTransactionId($paymentData['transactionId']);
        $payment
            ->setIsTransactionApproved(true)
            ->setShouldCloseParentTransaction(true)
            ->setIsTransactionClosed(true);

        if ($order->canInvoice()) {
            $invoice = $order->prepareInvoice();
            $invoice->register();
            $invoice->setTransactionId($paymentData['transactionId']);
            $order->addRelatedObject($invoice);
        }
        $order->addStatusHistoryComment('Zaksięgowano.', 'OK');
        $order->getResource()->save($order);
    }

    public function getConfigData($configPath)
    {
        return $this->scopeConfig->getValue(
            'payment/paybylink_payments/' . $configPath,
            \Magento\Store\Model\ScopeInterface::SCOPE_STORE
        );
    }
}
