<?php
include_once "transfergenerateresponse.class.php";
include_once "transfergeneraterequest.class.php";
include_once "transfernotification.class.php";

class RushPayMain {
    private $apiVersion;
    private $shopId;
    private $secret;
    private $algo;

    public function __construct($apiVersion, $shopId, $secret, $algo) {
        $this -> apiVersion = $apiVersion;
        $this -> shopId = $shopId;
        $this -> secret = $secret;
        $this -> algo = $algo;
    }

    public function getTransferNotification($data) {
        $decoded = json_decode($data, true);
        $transferNotification = new TransferNotification();
        $transferNotification -> transactionId = $decoded["transactionId"];
        $transferNotification -> control = $decoded["control"];
        $transferNotification -> email = $decoded["email"];
        $transferNotification -> amountPaid = $decoded["amountPaid"];
        $transferNotification -> notificationAttempt = $decoded["notificationAttempt"];
        $transferNotification -> paymentType = $decoded["paymentType"];
        $transferNotification -> apiVersion = $decoded["apiVersion"];
        $transferNotification -> signature = $decoded["signature"];
        if ($transferNotification -> checkSignature($this -> secret, $this -> algo)) {
            return $transferNotification;
        }
        return null;
        // return $transferNotification -> checkSignature($this -> secret, $this -> algo);
    }

    public function sendTransferGenerateRequest($r) {
        if (!$r instanceof TransferGenerateRequest) {
            throw new Exception("Object not TransferGenerateRequest");
        }
        if (!$r -> validateFields()) {
            throw new Exception("Fields not valid!");
        }
        $r -> setShopId($this -> shopId);
        $r -> calculateSignature($this -> secret, $this -> algo);
        $json = json_encode($r);
         $url = "https://secure.rushpay.pl/api/";
        $ch = curl_init($url . $this -> apiVersion . "/transfer/generate");
        curl_setopt($ch, CURLOPT_POSTFIELDS, $json);
        curl_setopt($ch, CURLOPT_HTTPHEADER, array("Content-Type: application/json"));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");

        $response =  curl_exec($ch);
        if(curl_errno($ch)){
            throw new Exception("CURL error: " . curl_error($ch));
        }
        $httpcode = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        if ($httpcode != 200) {
            if (($httpcode >= 400 && $httpcode <= 410) || ($httpcode >= 500 && $httpcode <= 510)) {
                $decoded = json_decode($response, true);
                throw new Exception("Error in transaction generating. Custom error: " . $decoded["errorCode"] . ": " . $decoded["error"] . " " . $response);
            } else {
                throw new Exception("Error in transaction generating. Code: " . $httpcode . " reason " . $response);

            }
        } else if ($httpcode == 200) {
            $decoded = json_decode($response, true);
            $responseData = new TransferGenerateResponse($decoded["url"], $decoded["transactionId"]);
            return $responseData;
        } else {
            throw new Exception("Unknown error " . $httpcode);
        }
    }
}
